
(function(O) {
	
   'use strict';
	
	String.prototype.format = String.prototype.f = function() {
	    var s = this,
	        i = arguments.length;

	    while (i--) {
	        s = s.replace(new RegExp('\\{' + i + '\\}', 'gm'), arguments[i]);
	    }
	    return s;
	};

	var msg = {

			"de" : {
			         "move": "bitte mache deinen Zug.",
			       	 "b_choose_move": "schwarz ist am Zug", 
			       	 "w_choose_move": "weiss ist am Zug", 			       	 
			       	 "black": "schwarz",
			       	 "white": "weiss",
			       	 "ai_thinking": "KI denkt nach...",
			       	 "new_game": "... neues Spiel starten",
			       	 "getting": "&nbsp;mit&nbsp;",
			       	 "draw" : "<b style='color: green'>Das Spiel endet unentschieden.</b>",	
			       	 "winner" : "<b style='color: red'>Der Gewinner ist ",
			       	 "discs": "&nbsp;Steinen",
			       	 "statistic": "<b>Ergebnisse:</b> Schwarz: {0}, Weiss: {1}, Unentschieden: {2}"
			},

			"en" : {
	                 "move": "Please, choose your move.",
			       	 "b_choose_move": "it's black's turn", 
			       	 "w_choose_move": "it's white's turn", 
	                 "black": "black",
	                 "white": "white",
	                 "ai_thinking": "Ai is thinking...",	
	                 "new_game": "... start a new Game",
	                 "getting": ",&nbsp;getting &nbsp; ",
	                 "draw":  "<b style='color: green'>game ends with a draw.</b>",
	                 "winner" : "<b style='color: red'>Winner is ",
	                 "discs": " &nbsp; Discs",
	                 "statistic": "<b>Results:</b> Black: {0}, White: {1}, Draw: {2}"
			}

	}
	
	let lang = O.lang;

	// UI {{{1

	function drawGameBoard(board, player, moves) {
		var ss = [];
		var attackable = [];
		moves.forEach(function(m) {
			if (!m.isPassingMove)
				attackable[O.ix(m.x, m.y)] = true;
		});

		ss.push('<table>');
		for (var y = -1; y < O.N; y++) {
			ss.push('<tr>');
			for (var x = -1; x < O.N; x++) {
				if (0 <= y && 0 <= x) {
					ss.push('<td class="');
					ss.push('cell');
					ss.push(' ');
					ss
							.push(attackable[O.ix(x, y)] ? player : board[O.ix(
									x, y)]);
					ss.push(' ');
					ss.push(attackable[O.ix(x, y)] ? 'attackable' : '');
					ss.push('" id="');
					ss.push('cell_' + x + '_' + y);
					ss.push('">');
					ss.push('<span class="disc"></span>');
					ss.push('</td>');
				} else if (0 <= x && y === -1) {
					ss.push('<th>' + String.fromCharCode('a'.charCodeAt(0) + x)
							+ '</th>');
				} else if (x === -1 && 0 <= y) {
					ss.push('<th>' + (y + 1) + '</th>');
				} else /* if (x === -1 && y === -1) */{
					ss.push('<th></th>');
				}
			}
			ss.push('</tr>');
		}
		ss.push('</table>');

		jQuery('#game-board').html(ss.join(''));
		if (player != "-") {
	
			if (player == 'black')
				jQuery('#current-player-name').text(msg[lang].b_choose_move );	
			else		
				jQuery('#current-player-name').text(msg[lang].w_choose_move );			
		}
	}

	function resetUI() {
		jQuery('#console').empty();
		jQuery('#message').empty();
	}

	function setUpUIToChooseMove(gameTree) {
		
	    jQuery('#message').text(msg[lang].move);

		gameTree.moves.forEach(function(m, i) {
			if (m.isPassingMove) {
				jQuery('#console').append(
						jQuery('<input type="button" class="button">').val(
								O.nameMove(m)).click(function() {
							shiftToNewGameTree(O.force(m.gameTreePromise));
						}));
			} else {
				jQuery('#cell_' + m.x + '_' + m.y).click(function() {
					shiftToNewGameTree(O.force(m.gameTreePromise));
				});
			}
		});
	}

	function setUpUIToReset() {
		resetGame();
		if (jQuery('#repeat-games:checked').length)
			startNewGame();
	}

	var minimumDelayForAI = 500; // milliseconds
	function chooseMoveByAI(gameTree, ai) {
		
		jQuery('#message').text(msg[lang].ai_thinking);

		setTimeout(function() {
			var start = Date.now();
			var newGameTree = O
					.force(ai.findTheBestMove(gameTree).gameTreePromise);
			var end = Date.now();
			var delta = end - start;
			setTimeout(function() {
				shiftToNewGameTree(newGameTree);
			}, Math.max(minimumDelayForAI - delta, 1));
		}, 1);
	}

	function showWinner(board) {
		
		var black_count = O.result(board);
		var white_count = 64 - O.result(board);
		
		var r = O.judge(board);	    

			jQuery('#message')
					.html( r === 0 ? msg[lang].draw : msg[lang].winner
									        + (r === 1 ? msg[lang].black : msg[lang].white)
											+ msg[lang].getting
											+ (r === 1 ? black_count : white_count )
											+  msg[lang].discs);
			jQuery('#current-player-name').text(msg[lang].new_game); 
		
	}

	var playerTable = {};

	function makePlayer(playerType) {
		if (playerType === 'human') {
			return setUpUIToChooseMove;
		} else {
			var ai = O.makeAI(playerType);
			return function(gameTree) {
				chooseMoveByAI(gameTree, ai);
			};
		}
	}

	function blackPlayerType() {
		return jQuery('#black-player-type').val();
	}

	function whitePlayerType() {
		return jQuery('#white-player-type').val();
	}

	function swapPlayerTypes() {
		var t = jQuery('#black-player-type').val();
		jQuery('#black-player-type').val(jQuery('#white-player-type').val())
				.change();
		jQuery('#white-player-type').val(t).change();
	}

	function shiftToNewGameTree(gameTree) {
		drawGameBoard(gameTree.board, gameTree.player, gameTree.moves);
		resetUI();
		if (gameTree.moves.length === 0) {
			showWinner(gameTree.board);
			recordStat(gameTree.board);
			//if (jQuery('#repeat-games:checked').length)
				showStat();
			setUpUIToReset();
		} else {
			playerTable[gameTree.player](gameTree);
		}
	}

	var stats = {};

	function recordStat(board) {
		var s = stats[[ blackPlayerType(), whitePlayerType() ]] || {
			b : 0,
			w : 0,
			d : 0
		};
		var r = O.judge(board);
		if (r === 1)
			s.b++;
		if (r === 0)
			s.d++;
		if (r === -1)
			s.w++;
		stats[[ blackPlayerType(), whitePlayerType() ]] = s;
	}

	function showStat() {
		var s = stats[[ blackPlayerType(), whitePlayerType() ]];
			jQuery('#stats').html(
					msg[lang].statistic.format(s.b, s.w, s.d));
	}

	function resetGame() {
		jQuery('#preference-pane :input:not(#repeat-games)').removeClass(
				'disabled').removeAttr('disabled');
	}

	function startNewGame() {
	  // jQuery('#preference-pane :input:not(#repeat-games)')
	  //	  .addClass('disabled') .attr('disabled', 'disabled');
		playerTable[O.BLACK] = makePlayer(blackPlayerType());
		playerTable[O.WHITE] = makePlayer(whitePlayerType());
		shiftToNewGameTree(O.makeInitialGameTree());
	}

	// Startup {{{1
	jQuery('#start-button').click(function() {
		startNewGame();
	});
	jQuery('#add-new-ai-button').click(function() {
		O.addNewAI();
	});
	jQuery('#swap-player-types-button').click(function() {
		swapPlayerTypes();
	});
	resetGame();
	drawGameBoard(O.makeInitialGameBoard(), '-', []);

	// }}}
})(othello);
// vim: expandtab softtabstop=2 shiftwidth=2 foldmethod=marker
